/*
 * Jagg Directory Anaylzer
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.io;

import net.chaosserver.beans.AsynchronousPropertyChangeSupport;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.io.File;
import java.io.IOException;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;


public class FileMetaData {
    public static final String PARENT_META_DATA_LOADED_PROPERTY =
        FileMetaData.class.getName() + ".parentMetaDataLoaded";
    public static final String LOADING_PROPERTY =
        FileMetaData.class.getName() + ".loading";
    public static final String PARENT_METADATA_LOAD_STATE_UNLOADED = "Unloaded";
    public static final String PARENT_METADATA_LOAD_STATE_LOADING = "Loading";
    public static final String PARENT_METADATA_LOAD_STATE_LOADED = "Loaded";
    protected File wrappedFile;
    protected String parentMetaDataLoadState =
        PARENT_METADATA_LOAD_STATE_UNLOADED;
    protected FileMetaData parentMetaData;
    protected Map childFileMetaDataMap;
    protected long size = -1;
    protected long length = -1;
    protected boolean fullload = false;
    protected PropertyChangeSupport propertyChangeSupport =
        new AsynchronousPropertyChangeSupport(this);

    public FileMetaData(File file, FileMetaData parentMetaData) {
        setWrappedFile(file);
        setParentMetaData(parentMetaData);
        childFileMetaDataMap = new HashMap();
    }

    public FileMetaData(String pathname) {
        this(new File(pathname));
    }

    public FileMetaData(File file) {
        this(file, (FileMetaData) null);
    }

    protected FileMetaData(File file, Map childMetaData) {
        this(file);
        childFileMetaDataMap.putAll(childMetaData);
    }

    /**
     * Setter for the wrappedFile property
     *
     * @param wrappedFile the new value of the wrappedFile property
     * @see #getWrappedFile
     */
    protected void setWrappedFile(File wrappedFile) {
        try {
            this.wrappedFile = wrappedFile.getCanonicalFile();
        } catch (IOException e) {
            throw new RuntimeException("wrapped: " + e);
        }
    }

    /**
     * Getter for the wrappedFile property
     *
     * @return the value of the wrappedFile property
     * @see #setWrappedFile
     */
    protected File getWrappedFile() {
        return this.wrappedFile;
    }

    /**
     * Getter for the parentMetaData property
     *
     * @return the value of the parentMetaData property
     * @see #setParentMetaData
     */
    protected void setParentMetaData(FileMetaData parentMetaData) {
        boolean parentMetaDataLoadedOldValue = isParentMetaDataLoaded();

        synchronized (parentMetaDataLoadState) {
            this.parentMetaData = parentMetaData;

            if (this.parentMetaData != null) {
                setParentMetaDataLoadState(PARENT_METADATA_LOAD_STATE_LOADED);
            }
        }
    }

    protected void setParentMetaDataLoadState(String parentMetaDataLoadState) {
        String oldParentMetaDataLoadState = this.parentMetaDataLoadState;
        this.parentMetaDataLoadState = parentMetaDataLoadState;

        propertyChangeSupport.firePropertyChange(
            PARENT_META_DATA_LOADED_PROPERTY, oldParentMetaDataLoadState,
            this.parentMetaDataLoadState);
    }

    public String getParentMetaDataLoadState() {
        return this.parentMetaDataLoadState;
    }

    public boolean isStateTransitioning() {
        boolean stateTransitioning = false;

        if (
            PARENT_METADATA_LOAD_STATE_LOADING.equals(
                    getParentMetaDataLoadState())) {
            stateTransitioning = true;
        }

        return stateTransitioning;
    }

    public void setLength(long length) {
    }

    public long getLength() {
        if (length < 0) {
            if (getWrappedFile().isDirectory()) {
                length = getDirectoryLength();
            } else {
                length = getWrappedFile().length();
            }
        }

        return length;
    }

    public long length() {
        return getLength();
    }

    public void setSize(long size) {
    }

    public long getSize() {
        if (size < 0) {
            if (getWrappedFile().isDirectory()) {
                long total = 0;
                File[] files = getWrappedFile().listFiles();

                if (files != null) {
                    for (int i = 0; i < files.length; i++) {
                        total += getChildMetaData(files[i]).size();
                    }
                }

                size = total;
            } else {
                size = 1;
            }
        }

        return size;
    }

    public long size() {
        return getSize();
    }

    public boolean isDirectory() {
        return getWrappedFile().isDirectory();
    }

    public String getName() {
        return getWrappedFile().getName();
    }

    public String getFullName() {
        String name = getWrappedFile().getName();

        try {
            name = getWrappedFile().getCanonicalPath();
        } catch (Exception e) {
        }

        return name;
    }

    public List getChildNames() {
        List childNames = new LinkedList();

        File[] files = getWrappedFile().listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                childNames.add(files[i].getName());
            }

            Collections.sort(childNames);
        }

        return childNames;
    }

    public FileMetaData getChildMetaData(String filename) {
        return getChildMetaData(new File(getWrappedFile(), filename));
    }

    public void reloadChildMetaData() {
        Map newChildFileMetaDataMap = new HashMap();
        File[] files = getWrappedFile().listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                FileMetaDataProgress.getInstance().setProcessing(files[i]);

                FileMetaData childMetaData = new FileMetaData(files[i], this);
                newChildFileMetaDataMap.put(
                    childMetaData.getName(), childMetaData);
            }
        }

        synchronized (childFileMetaDataMap) {
            childFileMetaDataMap = newChildFileMetaDataMap;
            fullload = true;
        }
    }

    public void loadChildMetaData() {
        File[] files = getWrappedFile().listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                FileMetaDataProgress.getInstance().setProcessing(files[i]);
                getChildMetaData(files[i]).getLength();
            }
        }

        fullload = true;
    }

    public Map getChildMetaData() {
        if (!fullload) {
            loadChildMetaData();
        }

        return this.childFileMetaDataMap;
    }

    public FileMetaData getChildMetaData(File file) {
        FileMetaData childMetaData;

        synchronized (childFileMetaDataMap) {
            childMetaData =
                (FileMetaData) childFileMetaDataMap.get(file.getName());

            if (childMetaData == null) {
                childMetaData = new FileMetaData(file, this);
                childFileMetaDataMap.put(
                    childMetaData.getName(), childMetaData);
            }
        }

        return childMetaData;
    }

    public boolean isParentMetaDataLoaded() {
        boolean parentMetaDataLoaded;

        if (
            PARENT_METADATA_LOAD_STATE_LOADED.equals(
                    getParentMetaDataLoadState())) {
            parentMetaDataLoaded = true;
        } else {
            parentMetaDataLoaded = false;
        }

        return parentMetaDataLoaded;
    }

    public void loadParentMetaDataBackground() {
        boolean shouldLoad = false;

        synchronized (parentMetaDataLoadState) {
            if (
                PARENT_METADATA_LOAD_STATE_UNLOADED.equals(
                        getParentMetaDataLoadState())) {
                shouldLoad = true;
                setParentMetaDataLoadState(PARENT_METADATA_LOAD_STATE_LOADING);
            }
        }

        if (shouldLoad) {
            ParentMetaDataLoader parentMetaDataLoader =
                new ParentMetaDataLoader(this);
            Thread thread = new Thread(parentMetaDataLoader);
            thread.start();
        }
    }

    public void loadParentMetaData() {
        loadParentMetaData(false);
    }

    public void loadParentMetaData(boolean loadAllChildren) {
        boolean shouldLoad = false;

        synchronized (parentMetaDataLoadState) {
            if (
                !PARENT_METADATA_LOAD_STATE_LOADED.equals(
                        getParentMetaDataLoadState())) {
                setParentMetaDataLoadState(PARENT_METADATA_LOAD_STATE_LOADING);
                shouldLoad = true;
            }
        }

        if (shouldLoad) {
            File parentFile = getWrappedFile().getParentFile();

            FileMetaDataProgress.getInstance().setProcessing(parentFile);

            Map childMap = new HashMap();
            childMap.put(getName(), this);

            FileMetaData parentFileMetaData =
                new FileMetaData(parentFile, childMap);

            if (loadAllChildren == true) {
                parentFileMetaData.loadChildMetaData();
            }

            setParentMetaData(parentFileMetaData);
        }
    }

    public FileMetaData getParentMetaData() {
        boolean shouldLoad = false;

        synchronized (parentMetaDataLoadState) {
            shouldLoad = true;
        }

        if (shouldLoad) {
            if (this.parentMetaData == null) {
                loadParentMetaData();
            }
        }

        return this.parentMetaData;
    }

    protected long getDirectoryLength() {
        long total = 0;
        File[] files = getWrappedFile().listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                FileMetaDataProgress.getInstance().setProcessing(files[i]);
                total += getChildMetaData(files[i]).length();
            }
        }

        return total;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    public String toString() {
        return this.getName();
    }

    protected class ParentMetaDataLoader implements Runnable {
        FileMetaData fileMetaData;

        public ParentMetaDataLoader(FileMetaData fileMetaData) {
            this.fileMetaData = fileMetaData;
        }

        public void run() {
            this.fileMetaData.loadParentMetaData(true);
        }
    }
}
